@@ -1,5 +1,6 @@
 from .periodic import *  # noqa
 from .topology_extractor import *  # noqa
 from .networkx import *  # noqa
-from .molecule_splitter import *  # noqa
+from .molecule_modifiers import *  # noqa
+from .atoms import *  # noqa
 from .conversion import *  # noqa
