@@ -3,6 +3,7 @@ Collapser Optimizer
 ===================
 
 #. :class:`.Collapser`
+#. :class:`.CollapserMC`
 
 Optimizer for collapsing enlarged topologies.
 
@@ -12,12 +13,13 @@ import logging
 from itertools import combinations
 import numpy as np
 from collections import defaultdict
-import matplotlib.pyplot as plt
 from scipy.spatial.distance import pdist
 import random
+import matplotlib.pyplot as plt
 import uuid
 import os
 import shutil
+from stk import PdbWriter
 
 from .optimizers import Optimizer
 from ..utilities import get_atom_distance
@@ -30,6 +32,10 @@ class Collapser(Optimizer):
     """
     Collapse stk.ConstructedMolecule to decrease enlarged bonds.
 
+    It is recommended to use the MCHammer version of this code with
+    :mod:`MCHammer` [1]_, where a much cleaner version is written.
+    The utilities `get_long_bond_ids` will help generate sub units.
+
     This optimizer aims to bring extended bonds closer together for
     further optimisation.
 
@@ -48,15 +54,18 @@ class Collapser(Optimizer):
         )
 
         # Perform collapser optimisation.
-        output_dir = f'cage_opt_{cage_name}_coll'
         optimizer = stko.Collapser(
-            output_dir=output_dir,
+            output_dir='test_coll',
             step_size=0.05,
             distance_cut=2.0,
             scale_steps=True,
         )
         cage1 = optimizer.optimize(mol=cage1)
 
+    References
+    ----------
+    .. [1] https://github.com/andrewtarzia/MCHammer
+
     """
 
     def __init__(
@@ -153,6 +162,24 @@ class Collapser(Optimizer):
 
         return new_position_matrix
 
+    def _get_new_cell_vectors(self, unit_cell, step, scales):
+        """
+        Get the unit cell vectors after collapse step.
+
+        """
+
+        vector_1 = unit_cell.get_vector_1()
+        vector_2 = unit_cell.get_vector_2()
+        vector_3 = unit_cell.get_vector_3()
+
+        max_scale = max(list(scales.values()))
+
+        vector_1 = vector_1 - vector_1*step*max_scale
+        vector_2 = vector_2 - vector_2*step*max_scale
+        vector_3 = vector_3 - vector_3*step*max_scale
+
+        return vector_1, vector_2, vector_3
+
     def _get_bb_vectors(self, mol, bb_atom_ids):
         """
         Get the building block to COM vectors.
@@ -289,14 +316,14 @@ class Collapser(Optimizer):
             step_no += 1
             mol = mol.with_position_matrix(new_pos)
             mol.write(
-                os.path.join(output_dir, f'collapsed_rev.mol')
+                os.path.join(output_dir, 'collapsed_rev.mol')
             )
 
-        out_file = os.path.join(output_dir, f'collapser.out')
+        out_file = os.path.join(output_dir, 'collapser.out')
         with open(out_file, 'w') as f:
             f.write(
-                f"Collapser algorithm.\n"
-                f"====================\n"
+                "Collapser algorithm.\n"
+                "====================\n"
                 f"Step size: {self._step_size}\n"
                 f"Scale steps?: {self._scale_steps}\n"
                 f"Distance cut: {self._distance_cut}\n"
@@ -307,14 +334,156 @@ class Collapser(Optimizer):
             )
         return mol
 
+    def p_optimize(self, mol, unit_cell):
+        """
+        Optimize `mol`.
+
+        Parameters
+        ----------
+        mol : :class:`stk.ConstructedMolecule`
+            The molecule to be optimized.
+
+        unit_cell : :class:`.UnitCell`
+            The cell to be optimized.
+
+        Returns
+        -------
+        mol : :class:`stk.ConstructedMolecule`
+            The optimized molecule.
+
+        unit_cell : :class:`.UnitCell`
+            The optimized cell.
+
+        """
+
+        # Handle output dir.
+        if self._output_dir is None:
+            output_dir = str(uuid.uuid4().int)
+        else:
+            output_dir = self._output_dir
+        output_dir = os.path.abspath(output_dir)
+
+        if os.path.exists(output_dir):
+            shutil.rmtree(output_dir)
+        os.mkdir(output_dir)
+
+        bb_atom_ids = defaultdict(list)
+        for i in mol.get_atom_infos():
+            bb_atom_ids[i.get_building_block_id()].append(
+                i.get_atom().get_id()
+            )
+
+        # Translate each building block along bb_COM_vectors by a
+        # distance `step`. I.e. `step` is the proportion of the
+        # bb_COM_vectors that the building block is moved.
+        step_no = 0
+        step = self._step_size
+        while not self._has_short_contacts(mol):
+            # Update each step the building block vectors and distance.
+            bb_cent_vectors, bb_cent_scales = self._get_bb_vectors(
+                mol=mol,
+                bb_atom_ids=bb_atom_ids
+            )
+
+            new_pos = self._get_new_position_matrix(
+                mol=mol,
+                step=step,
+                vectors=bb_cent_vectors,
+                scales=bb_cent_scales,
+            )
+            step_no += 1
+            mol = mol.with_position_matrix(new_pos)
+            new_vector_1, new_vector_2, new_vector_3 = (
+                self._get_new_cell_vectors(
+                    unit_cell=unit_cell,
+                    step=step,
+                    scales=bb_cent_scales,
+                )
+            )
+            unit_cell = unit_cell.with_cell_from_vectors(
+                vector_1=new_vector_1,
+                vector_2=new_vector_2,
+                vector_3=new_vector_3,
+            )
+            PdbWriter().write(
+                molecule=mol,
+                path=os.path.join(
+                    output_dir, f'collapsed_{step_no}.pdb'
+                ),
+                periodic_info=unit_cell,
+            )
+
+        bb_cent_vectors, bb_cent_scales = self._get_bb_vectors(
+            mol=mol,
+            bb_atom_ids=bb_atom_ids
+        )
+
+        # Check that we have not gone too far.
+        min_dist = min(
+            dist for dist in self._get_inter_bb_distance(mol)
+        )
+        if min_dist < self._distance_cut / 2:
+            # Revert to half the previous step if we have.
+            step = -(self._step_size/2)
+            new_pos = self._get_new_position_matrix(
+                mol=mol,
+                step=step,
+                vectors=bb_cent_vectors,
+                scales=bb_cent_scales
+            )
+            step_no += 1
+            mol = mol.with_position_matrix(new_pos)
+            new_vector_1, new_vector_2, new_vector_3 = (
+                self._get_new_cell_vectors(
+                    unit_cell=unit_cell,
+                    step=step,
+                    scales=bb_cent_scales,
+                )
+            )
+            unit_cell = unit_cell.with_cell_from_vectors(
+                vector_1=new_vector_1,
+                vector_2=new_vector_2,
+                vector_3=new_vector_3,
+            )
+            PdbWriter().write(
+                molecule=mol,
+                path=os.path.join(
+                    output_dir, 'collapsed_rev.pdb'
+                ),
+                periodic_info=unit_cell,
+            )
+
+        out_file = os.path.join(output_dir, 'collapser.out')
+        with open(out_file, 'w') as f:
+            f.write(
+                "Collapser algorithm.\n"
+                "====================\n"
+                f"Step size: {self._step_size}\n"
+                f"Scale steps?: {self._scale_steps}\n"
+                f"Distance cut: {self._distance_cut}\n"
+                f"====================\n"
+                f"Steps run: {step_no}\n"
+                f"Minimum inter-bb distance: {min_dist}\n"
+                f"====================\n"
+            )
+        return mol, unit_cell
+
 
 class CollapserMC(Collapser):
     """
     Collapse molecule to decrease enlarged bonds using MC algorithm.
 
+    It is recommended to use the MCHammer version of this code with
+    :mod:`MCHammer` [1]_, where a much cleaner version is written.
+    The utilities `get_long_bond_ids` will help generate sub units.
+
     Smarter optimisation than Collapser using simple Monte Carlo
     algorithm to perform rigid translations of building blocks.
 
+    References
+    ----------
+    .. [1] https://github.com/andrewtarzia/MCHammer
+
     """
 
     def __init__(
@@ -484,7 +653,6 @@ class CollapserMC(Collapser):
         Define an arbitrary parabolic bond potential.
 
         This potential has no relation to an empircal forcefield.
-
         """
 
         potential = (distance - self._target_bond_length) ** 2
@@ -497,7 +665,6 @@ class CollapserMC(Collapser):
         Define an arbitrary repulsive nonbonded potential.
 
         This potential has no relation to an empircal forcefield.
-
         """
 
         return (
@@ -510,9 +677,11 @@ class CollapserMC(Collapser):
 
         # Get all pairwise distances.
         pair_dists = pdist(mol.get_position_matrix())
-        non_bonded_potential = self._non_bond_potential(pair_dists)
+        nonbonded_potential = np.sum(
+            self._non_bond_potential(pair_dists)
+        )
 
-        return sum(non_bonded_potential)
+        return nonbonded_potential
 
     def _compute_potential(self, mol, long_bond_infos):
 
@@ -539,16 +708,6 @@ class CollapserMC(Collapser):
 
         return mol.with_position_matrix(new_position_matrix)
 
-    def _rotate_atoms_onto_vector(
-        self,
-        mol,
-        atom_ids,
-        start_vector,
-        target_vector,
-        axis
-    ):
-        raise NotImplementedError()
-
     def _test_move(self, curr_pot, new_pot):
 
         if new_pot < curr_pot:
@@ -593,7 +752,7 @@ class CollapserMC(Collapser):
         ax.axhline(y=self._target_bond_length, c='r')
         fig.tight_layout()
         fig.savefig(
-            os.path.join(output_dir, f'maxd_vs_step.pdf'),
+            os.path.join(output_dir, 'maxd_vs_step.pdf'),
             dpi=360,
             bbox_inches='tight'
         )
@@ -609,7 +768,7 @@ class CollapserMC(Collapser):
         ax.legend(fontsize=16)
         fig.tight_layout()
         fig.savefig(
-            os.path.join(output_dir, f'pot_vs_step.pdf'),
+            os.path.join(output_dir, 'pot_vs_step.pdf'),
             dpi=360,
             bbox_inches='tight'
         )
@@ -649,10 +808,6 @@ class CollapserMC(Collapser):
         if len(long_bond_infos) == 0:
             return mol
 
-        # Get bb atom ids and bb centroids.
-        bb_atom_ids = self._get_bb_atom_ids(mol)
-        bb_centroids = self._get_bb_centroids(mol, bb_atom_ids)
-
         # Define bb centroid - long bond atom vectors.
         # These are to be maintained during optimisation.
         # centroid_to_lb_vectors = self._get_cent_to_lb_vector(
@@ -667,9 +822,9 @@ class CollapserMC(Collapser):
             long_bond_infos=long_bond_infos
         )
 
-        with open(os.path.join(output_dir, f'coll.out'), 'w') as f:
+        with open(os.path.join(output_dir, 'coll.out'), 'w') as f:
             f.write(self._output_top_lines())
-            mol.write(os.path.join(output_dir, f'coll_0.mol'))
+            mol.write(os.path.join(output_dir, 'coll_0.mol'))
             steps = [0]
             passed = []
             spots = [system_potential]
